#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 19:26, 24/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
# -------------------------------------------------------------------------------------------------------%

import pkg_resources
from pandas import read_csv
from copy import deepcopy
from numpy import sum, exp, cos, sin, sqrt, e, pi, abs, round, log, power, fill_diagonal, zeros, reshape


class BasicFunction:
    def __init__(self, cec_type="cec2013", dimensions=(2, 5, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100)):
        self.cec_type = cec_type
        self.dimensions = dimensions
        self.CURRENT_PATH = pkg_resources.resource_filename("opfunu", "cec/" + cec_type + "/")
        self.SUPPORT_PATH_DATA = pkg_resources.resource_filename("opfunu", "cec/" + cec_type + "/support_data/")

    def load_shift_data__(self, data_file=None):
        data = read_csv(self.SUPPORT_PATH_DATA + data_file, delimiter='\s+', index_col=False, header=None)
        return data.values.reshape((-1))

    def load_matrix_data__(self, data_file=None):
        data = read_csv(self.SUPPORT_PATH_DATA + data_file, delimiter='\s+', index_col=False, header=None)
        return data.values

    def check_dimensions(self, dimension):
        if dimension not in self.dimensions:
            print("{} only support problem size: {}".format(self.cec_type.upper(), self.dimensions))
            exit(0)

    def ackley__(self, solution=None):
        solution = solution.reshape((-1))
        return -20 * exp(-0.2 * sqrt(sum(solution ** 2) / len(solution))) - exp(sum(cos(2 * pi * solution)) / len(solution)) + 20 + e

    def bent_cigar__(self, solution=None):
        solution = solution.reshape((-1))
        return solution[0] ** 2 + 10 ** 6 * sum(solution[1:] ** 2)

    def discus__(self, solution=None):
        solution = solution.reshape((-1))
        return 10 ** 6 * solution[0] ** 2 + sum(solution[1:] ** 2)

    def weierstrass__(self, solution=None, a=0.5, b=3, k_max=20):
        solution = solution.reshape((-1))
        result = 0.0
        for i in range(0, len(solution)):
            t1 = sum([a ** k * cos(2 * pi * b ** k * (solution[i] + 0.5)) for k in range(0, k_max)])
            result += t1
        t2 = len(solution) * sum([a ** k * cos(2 * pi * b ** k * 0.5) for k in range(0, k_max)])
        return result - t2

    def katsuura__(self, solution=None):
        solution = solution.reshape((-1))
        result = 1.0
        for i in range(0, len(solution)):
            t1 = sum([abs(2 ** j * solution[i] - round(2 ** j * solution[i])) / 2 ** j for j in range(1, 33)])
            result *= (1 + (i + 1) * t1) ** (10.0 / len(solution) ** 1.2)
        return (result - 1) * 10 / len(solution) ** 2

    def happy_cat__(self, solution=None):
        solution = solution.reshape((-1))
        return (abs(sum(solution ** 2) - len(solution))) ** 0.25 + (0.5 * sum(solution ** 2) + sum(solution)) / len(solution) + 0.5

    def hgbat__(self, solution=None):
        solution = solution.reshape((-1))
        return (abs(sum(solution ** 2) ** 2 - sum(solution) ** 2)) ** 0.5 + (0.5 * sum(solution ** 2) + sum(solution)) / len(solution) + 0.5

    def rosenbrock__(self, solution=None):
        solution = solution.reshape((-1))
        result = 0.0
        for i in range(len(solution) - 1):
            result += 100 * (solution[i] ** 2 - solution[i + 1]) ** 2 + (solution[i] - 1) ** 2
        return result

    def rastrigin__(self, solution=None):
        solution = solution.reshape((-1))
        return sum(solution ** 2 - 10 * cos(2 * pi * solution) + 10)

    def elliptic__(self, solution=None):
        solution = solution.reshape((-1))
        result = 0
        for i in range(len(solution)):
            result += (10 ** 6) ** (i / (len(solution) - 1)) * solution[i] ** 2
        return result

    def modified_schwefel__(self, solution=None):
        solution = solution.reshape((-1))
        z = solution + 4.209687462275036e+002
        result = 418.9829 * len(solution)
        for i in range(0, len(solution)):
            if z[i] > 500:
                result -= (500 - z[i] % 500) * sin(sqrt(abs(500 - z[i] % 500))) - (z[i] - 500) ** 2 / (10000 * len(solution))
            elif z[i] < -500:
                result -= (z[i] % 500 - 500) * sin(sqrt(abs(z[i] % 500 - 500))) - (z[i] + 500) ** 2 / (10000 * len(solution))
            else:
                result -= z[i] * sin(abs(z[i]) ** 0.5)
        return result

    def expanded_scaffer__(self, solution=None):
        solution = solution.reshape((-1))
        def __xy__(x, y):
            return 0.5 + (sin(sqrt(x ** 2 + y ** 2)) ** 2 - 0.5) / (1 + 0.001 * (x ** 2 + y ** 2)) ** 2

        result = __xy__(solution[-1], solution[0])
        for i in range(0, len(solution) - 1):
            result += __xy__(solution[i], solution[i + 1])
        return result

    def griewank__(self, solution=None):
        solution = solution.reshape((-1))
        result = sum(solution ** 2) / 4000
        temp = 1.0
        for i in range(len(solution)):
            temp *= cos(solution[i] / sqrt(i + 1))
        return result - temp + 1

    def expanded_griewank__(self, solution=None):
        solution = solution.reshape((-1))
        def __rosenbrock__(x=None, y=None):
            return 100 * (x ** 2 - y) ** 2 + (x - 1) ** 2

        def __griewank__(z=None):
            return z ** 2 / 4000 - cos(z / sqrt(1)) + 1

        result = __griewank__(__rosenbrock__(solution[-1], solution[0]))
        for i in range(0, len(solution) - 1):
            result += __griewank__(__rosenbrock__(solution[i], solution[i + 1]))
        return result

    def different_powers__(self, solution=None):
        solution = solution.reshape((-1))
        result = 0.0
        for i in range(0, len(solution)):
            result += power(abs(solution[i]), 2 + 4 * i / (len(solution) - 1))
        return sqrt(result)

    def schaffers_f7__(self, solution=None):
        solution = solution.reshape((-1))
        result = 0.0
        for i in range(0, len(solution) - 1):
            result += power(solution[i] ** 2 + solution[i + 1] ** 2, 0.25) * (1 + sin(50 * sqrt(solution[i] ** 2 + solution[i + 1] ** 2) ** 0.2) ** 2)
        return (result / (len(solution) - 1)) ** 2

    def osz_func__(self, solution=None):
        solution = solution.reshape((-1))
        solution_new = deepcopy(solution)
        for idx in range(0, len(solution)):
            if idx == 0 or idx == len(solution) - 1:
                c1 = 5.5
                c2 = 3.1
                x_sign = 1
                x_star = log(abs(solution[idx]))
                if solution[idx] < 0:
                    x_sign = -1
                elif solution[idx] > 0:
                    c1 = 10
                    c2 = 7.9
                else:
                    x_sign = 0
                    x_star = 0
                solution_new[idx] = x_sign * exp(x_star + 0.049 * (sin(c1 * x_star) + sin(c2 * x_star)))
        return solution_new

    def asy_func__(self, solution=None, beta=0.5):
        solution = solution.reshape((-1))
        solution_new = deepcopy(solution)
        for idx in range(len(solution)):
            if solution[idx] > 0:
                solution_new[idx] = power(solution[idx], 1 + beta * idx * sqrt(solution[idx]) / (len(solution) - 1))
        return solution_new

    def create_diagonal_matrix__(self, size=None, alpha=10):
        matrix = zeros((size, size), float)
        temp = [power(alpha, i / (2 * (size - 1))) for i in range(0, size)]
        fill_diagonal(matrix, temp)
        return matrix



#!/usr/bin/env python                                                                                   #
# ------------------------------------------------------------------------------------------------------#
# Created by "Thieu Nguyen" at 02:54, 06/12/2019                                                        #
#                                                                                                       #
#       Email:      nguyenthieu2102@gmail.com                                                           #
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  #
#       Github:     https://github.com/thieunguyen5991                                                  #
#-------------------------------------------------------------------------------------------------------#

import numpy as np

class Functions:
    """
        This class of functions is belongs to 2-dimensional space
    """

    def _ackley_n2__(self, solution=None):
        """
        Class: unimodal, convex, differentiable, non-separable
        Global: one global minimum fx = -200, at [0, 0]

        @param solution: A numpy array include 2 items like: [10, 22]
        """
        n = len(solution)
        assert (n == 2, 'Ackley N. 2 function is only defined on a 2D space.')
        return -200*np.exp(-0.2*np.sqrt(np.sum(solution**2)))


    def _ackley_n3__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, non-separable
        Global: one global minimum fx = −195.629028238419, at [±0.682584587365898,−0.36075325513719]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/ackleyn3fcn.html

        @param solution: A numpy array include 2 items like: [10, 22]
        """
        d = len(solution)
        assert (d == 2, 'Ackley N. 3 function is only defined on a 2D space.')
        return -200*np.exp(-0.2*np.sqrt(np.sum(solution**2))) + 5*np.exp(np.cos(3*solution[0]) + np.sin(3*solution[1]))


    def _adjiman__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, non-separable
        Global: if x in [-1, 2], y in [-1, 1] cube => global min fx = -2.02181, at [0, 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/adjimanfcn.html

        @param solution: A numpy array include 2 items like: [10, 22]
        """
        d = len(solution)
        assert (d == 2, 'Adjiman function is only defined on a 2D space.')
        return np.cos(solution[0]) * np.sin(solution[1]) - solution[0] / (solution[1]**2 + 1)

    def _bartels_conn__(self, solution=None):
        """
        Class: multimodal, non-convex, non-differentiable, non-separable
        Global: one global minimum fx = 1, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/bartelsconnfcn.html

        @param solution: A numpy array include 2 items like: [10, 22]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Bartels conn function is only defined on a 2D space.')
        return np.abs(solution[0]**2 + solution[1]**2 + solution[0] * solution[1]) + np.abs(np.sin(solution[0])) + \
               np.abs(np.cos(solution[1]))

    def _beale__(self, solution=None):
        """
        Class: multimodal, non-convex, continuous
        Global: one global minimum fx = 0, at [3, 0.5]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/bealefcn.html

        @param solution: A numpy array include 2 items in range: [-4.5, 4.5], [-4.5, 4.5]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Beale function is only defined on a 2D space.')
        return (1.5-solution[0]+solution[0]*solution[1])**2 + (2.25-solution[0]+solution[0]*solution[1]**2)**2 +\
               (2.625-solution[0]+solution[0]*solution[1]**3)**2

    def _bird__(self, solution=None):
        """
        Class: multimodal, non-convex, non-separable, differentiable
        Global: 2 global minimum fx= -106.764537, at ( 4.70104 , 3.15294 ) and ( − 1.58214 , − 3.13024 ) .
        Link: http://benchmarkfcns.xyz/benchmarkfcns/birdfcn.html

        @param solution: A numpy array include 2 items in range: [-2pi, 2pi], [-2pi, 2pi]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Bird function is only defined on a 2D space.')
        return np.sin(solution[0])*np.exp((1 - np.cos(solution[1]))**2) + \
            np.cos(solution[1]) * np.exp( (1-np.sin(solution[0]))**2 ) + (solution[0] - solution[1])**2

    def _bohachevskyn_n1__(self, solution=None):
        """
        Class: unimodal, convex, continuous
        Global: global minimum fx= 0, at ( 0, 0 )
        Link: http://benchmarkfcns.xyz/benchmarkfcns/bohachevskyn1fcn.html

        @param solution: A numpy array include 2 items in range: [-100, 100], [-100, 100]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Bohachevskyn N.1 function is only defined on a 2D space.')
        return solution[0]**2 + 2*solution[1]**2 - 0.3*np.cos(3*solution[0]*np.pi) - 0.4*np.cos(4*solution[1]*np.pi) + 0.7


    def _bohachevskyn_n2__(self, solution=None):
        """
        Class: multi-modal, non-convex, non-separable, differentiable
        Global: global minimum fx= 0, at ( 0, 0 )
        Link: http://benchmarkfcns.xyz/benchmarkfcns/bohachevskyn2fcn.html

        @param solution: A numpy array include 2 items in range: [-100, 100], [-100, 100]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Bohachevskyn N.2 function is only defined on a 2D space.')
        return solution[0] ** 2 + 2 * solution[1] ** 2 - 0.3 * np.cos(3 * solution[0] * np.pi) * np.cos(4 * solution[1] * np.pi) + 0.3


    def _booth__(self, solution=None):
        """
        Class: unimodal, convex, non-separable, differentiable, continuous
        Global: one global minimum fx= 0, at ( 1, 3 )
        Link: http://benchmarkfcns.xyz/benchmarkfcns/boothfcn.html

        @param solution: A numpy array include 2 items in range: [-10, 10], [-10, 10]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Brooth function is only defined on a 2D space.')
        return  (solution[0]+2*solution[1]-7)**2 + (2*solution[0]+solution[1]-5)**2


    def _brent__(self, solution=None):
        """
        Class: unimodal, convex, non-separable, differentiable
        Global: one global minimum fx= e^(-200), at ( -10 -10 )
        Link: http://benchmarkfcns.xyz/benchmarkfcns/brentfcn.html

        @param solution: A numpy array include 2 items in range: [-20, 0], [-20, 0]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Brent function is only defined on a 2D space.')
        return (solution[0]+10)**2 + (solution[1]+10)**2 + np.exp(-solution[0]**2-solution[1]**2)


    def _bukin_n6__(self, solution=None):
        """
        Class: multimodal, convex, non-separable, non-differentiable, continuous
        Global: one global minimum fx= 0, at ( -10, 1)
        Link: http://benchmarkfcns.xyz/benchmarkfcns/bukinn6fcn.html

        @param solution: A numpy array include 2 items in range: [-15, -5], [-3, 3]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Bukin N.6 function is only defined on a 2D space.')
        return 100*np.sqrt(np.abs(solution[1] - 0.01*solution[0]**2)) + 0.01*np.abs(solution[0]+10)


    def _cross_in_tray__(self, solution=None):
        """
        Class: multimodal, non-convex, non-separable, non-differentiable, continuous
        Global: 4 global minimum fx= -2.06261218, at (±1.349406685353340,±1.349406608602084)
        Link: http://benchmarkfcns.xyz/benchmarkfcns/crossintrayfcn.html

        @param solution: A numpy array include 2 items in range: [-10, 10], [-10, 10]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Bukin N.6 function is only defined on a 2D space.')
        t1 = np.exp( np.abs(100 - np.sqrt(np.sum(solution**2))/np.pi ) )
        t2 = np.sin(solution[0]) * np.cos(solution[1])
        return -0.0001*(np.abs(t1*t2) + 1)**0.1


    def _deckkers_aarts__(self, solution=None):
        """
        Class: multimodal, non-convex, non-separable, differentiable, continuous
        Global: 1 global minimum fx = −24771.09375, at ( 0 , ± 15 ) .
        Link: http://benchmarkfcns.xyz/benchmarkfcns/deckkersaartsfcn.html

        @param solution: A numpy array include 2 items in range: [-20, 20], [-20, 20]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Deckkers Aarts function is only defined on a 2D space.')
        t1 = solution[0]**2
        t2 = solution[1]**2
        return 10**5*t1 + t2 - (t1 + t2)**2 + 10**(-5) * (t1 + t2)**4


    def _drop_wave__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous
        Global: 1 global minimum fx = −1 at ( 0 , 0 ) .
        Link: http://benchmarkfcns.xyz/benchmarkfcns/dropwavefcn.html

        @param solution: A numpy array include 2 items in range: [-5.2, 5.2], [-5.2, 5.2]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Drop wave function is only defined on a 2D space.')
        return -(1+np.cos(12*np.sqrt(np.sum(solution*2)))) / (0.5 * np.sum(solution**2) + 2)


    def _easom__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous, differentiable, separable
        Global: 1 global minimum fx = −1 at ( pi, pi ) .
        Link: http://benchmarkfcns.xyz/benchmarkfcns/easomfcn.html

        @param solution: A numpy array include 2 items in range: [-100, 100], [-100, 100]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Easom function is only defined on a 2D space.')
        return -np.cos(solution[0])*np.cos(solution[1])*np.exp(-(solution[0] - np.pi)**2 - (solution[1] - np.pi)**2)


    def _egg_crate__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous, differentiable, separable
        Global: global minimum fx = 0 at ( 0, 0 ) .
        Link: http://benchmarkfcns.xyz/benchmarkfcns/eggcratefcn.html

        @param solution: A numpy array include 2 items in range: [-5, 5]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Egg Crate function is only defined on a 2D space.')
        return np.sum(solution**2) + 25 * (np.sin(solution[0])**2 + np.sin(solution[1])**2)


    def _goldstein_price__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous, differentiable, non-separable
        Global: global minimum fx = 3 at ( 0, -1 ) .
        Link: http://benchmarkfcns.xyz/benchmarkfcns/goldsteinpricefcn.html

        @param solution: A numpy array include 2 items in range: [-2, 2]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Goldstein price function is only defined on a 2D space.')
        t1 = 18 - 32*solution[0] + 12*solution[0]**2 + 4*solution[1] - 36*solution[0]*solution[1] + 27*solution[1]**2
        t2 = 19 - 14*solution[0]+3*solution[0]**2 - 14*solution[1] + 6*solution[0]*solution[1] + 3*solution[1]**2
        t3 = (np.sum(solution) + 1)**2
        return (1+t3*t2) * (30 + (2*solution[0]-3*solution[1])**2 * t1)


    def _himmelblau__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous
        Global: 4 global optima, fx = 0 at (3, 2), (-2.85118, 3.283186), (−3.779310,−3.283186), (3.584458,−1.848126)

        Link: http://benchmarkfcns.xyz/benchmarkfcns/himmelblaufcn.html

        @param solution: A numpy array include 2 items in range: [-6, 6]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Himmelblau function is only defined on a 2D space.')
        return (solution[0]**2+solution[1]-11)**2 + (solution[0] + solution[1]**2 - 7)**2


    def _holder_table__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous, non-differentiable, non-separable
        Global: 4 global optima, fx = -19.2085 at (±8.05502,±9.66459)

        Link: http://benchmarkfcns.xyz/benchmarkfcns/holdertablefcn.html

        @param solution: A numpy array include 2 items in range: [-10, 10]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Holder Table function is only defined on a 2D space.')
        return -np.abs(np.sin(solution[0])*np.cos(solution[1])*np.abs(1 - np.sqrt(np.sum(solution**2))/np.pi))


    def _keane__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous, differentiable, non-separable
        Global: 2 global optima, fx = 0.673667521146855 at (1.393249070031784,0), (0,1.393249070031784)

        Link: http://benchmarkfcns.xyz/benchmarkfcns/kealefcn.html

        @param solution: A numpy array include 2 items in range: [0, 10]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Keane function is only defined on a 2D space.')
        return -np.sin(solution[0]-solution[1])**2 * np.sin(solution[0] + solution[1])**2 / np.sqrt(np.sum(solution**2))


    def _leon__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous, differentiable, non-separable
        Global: 1 global optima, fx = 0 at [0, 10]

        Link: http://benchmarkfcns.xyz/benchmarkfcns/leonfcn.html

        @param solution: A numpy array include 2 items in range: [0, 10]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Leon function is only defined on a 2D space.')
        return 100*(solution[1]-solution[0]**3)**2 + (1-solution[0])**2


    def _levi_n13__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous, differentiable, non-separable
        Global: 1 global optima, fx = 0 at [1, 1]

        Link: http://benchmarkfcns.xyz/benchmarkfcns/levin13fcn.html

        @param solution: A numpy array include 2 items in range: [-10, 10]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Levi N.13 function is only defined on a 2D space.')
        return np.sin(3*solution[0]*np.pi)**2 + (solution[0]-1)**2*(1+np.sin(3*solution[1]**np.pi)**2) +\
               (solution[1]-1)**2*(1 + np.sin(2*solution[1]*np.pi)**2)


    def _matyas__(self, solution=None):
        """
        Class: uni-modal, convex, continuous, differentiable, non-separable
        Global: 1 global optima, fx = 0 at [0, 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/matyasfcn.html

        @param solution: A numpy array include 2 items in range: [-10, 10]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Matyas function is only defined on a 2D space.')
        return 0.26*np.sum(solution**2) - 0.48*solution[0]*solution[1]


    def _mc_cormick__(self, solution=None):
        """
        Class: multi-modal, convex, continuous, differentiable, non-scalable
        Global: 1 global optima, fx = -1.9133 at [-0.547, -1.547]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/mccormickfcn.html

        @param solution: A numpy array include 2 items in range: [-1.5, 4], [-3, 3]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Mc Cormick function is only defined on a 2D space.')
        return np.sin(solution[0]+solution[1]) + (solution[0] - solution[1])**2 - 1.5*solution[0] + 2.5*solution[1] + 1


    def _schaffer_n1__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous, differentiable, non-separable
        Global: 1 global optima, fx = 0 at [0, 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schaffern1fcn.html

        @param solution: A numpy array include 2 items in range: [-100, 100]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Scheffer N.1 function is only defined on a 2D space.')
        return 0.5 + (np.sin(np.sum(solution**2)**2)**2 - 0.5) / (1 + 0.001*np.sum(solution**2))**2


    def _schaffer_n2__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous, differentiable, non-separable
        Global: 1 global optima, fx = 0 at [0, 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schaffern2fcn.html

        @param solution: A numpy array include 2 items in range: [-100, 100]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Scheffer N.2 function is only defined on a 2D space.')
        return 0.5 + (np.sin(solution[0]**2 - solution[1]**2)**2 - 0.5) / (1 + 0.001*np.sum(solution**2))**2


    def _schaffer_n3__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous, differentiable, non-separable
        Global: 1 global optima, fx = 0.00156685 at [0, 1.253115]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schaffern3fcn.html

        @param solution: A numpy array include 2 items in range: [-100, 100]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Scheffer N.3 function is only defined on a 2D space.')
        return 0.5 + (np.sin(np.cos(np.abs( solution[0]**2 - solution[1]**2 ))) - 0.5) / (1 + 0.001*np.sum(solution**2))**2


    def _schaffer_n4__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous, differentiable, non-separable
        Global: 1 global optima, fx = 0.292579 at [0, 1.253115]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schaffern4fcn.html

        @param solution: A numpy array include 2 items in range: [-100, 100]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Scheffer N.4 function is only defined on a 2D space.')
        return 0.5 + (np.cos(np.sin(np.abs( solution[0]**2 - solution[1]**2 ))) - 0.5) / (1 + 0.001*np.sum(solution**2))**2


    def _three_hump_camel__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous, differentiable, non-separable
        Global: 1 global optima, fx = 0 at [0, 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/threehumpcamelfcn.html

        @param solution: A numpy array include 2 items in range: [-5, 5]
        @return: fx
        """
        d = len(solution)
        assert (d == 2, 'Scheffer N.3 function is only defined on a 2D space.')
        return 2*solution[0]**2 - 1.05*solution[0]**4 + solution[0]**6/6 + solution[0]*solution[1] + solution[1]**2






#!/usr/bin/env python                                                                                   #
# ------------------------------------------------------------------------------------------------------#
# Created by "Thieu Nguyen" at 17:44, 06/12/2019                                                        #
#                                                                                                       #
#       Email:      nguyenthieu2102@gmail.com                                                           #
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  #
#       Github:     https://github.com/thieunguyen5991                                                  #
#-------------------------------------------------------------------------------------------------------#

import numpy as np

class Functions:
    """
        This class of functions is belongs to n-dimensional space
    """

    def _ackley__(self, solution=None, a=20, b=0.2, c=2 * np.pi):
        """
        Class: multimodal, continuous, non-convex, differentiable, n-dimensional space.
        Global: one global minimum fx = 0, at [0, 0,...0]

        @param solution: A numpy array like: [1, 2, 10, 4, ...]
        @return: fx
        """
        result1 = np.sum(solution ** 2)
        result2 = np.sum(np.cos(c * solution))
        lin = 1 / len(solution)
        return -a * np.exp(-b * np.sqrt(lin * result1)) - np.exp(lin * result2) + a + np.exp(1)


    def _ackley_n4__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, non-separable, n-dimensional space.
        Global: on 2-d space, 1 global min fx = -4.590101633799122, at [−1.51, −0.755]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/ackleyn4fcn.html

        @param solution: A numpy array include 2 items like: [-35, 35, -35, ...]
        """
        d = len(solution)
        score = 0.0
        for i in range(0, d-1):
            score += ( np.exp(-0.2*np.sqrt(solution[i]**2 + solution[i+1]**2)) + 3*(np.cos(2*solution[i]) + np.sin(2*solution[i+1])) )
        return score


    def _alpine_n1__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, non-separable, n-dimensional space.
        Global: one global minimum fx = 0, at [0, 0,...0]

        @param solution: A numpy array like: [1, 2, 10, 4, ...]
        @return: fx
        """
        return np.sum(np.abs(solution*np.sin(solution) + 0.1 * solution))


    def _alpine_n2__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, non-separable, n-dimensional space.
        Global: one global minimum fx = 2.808^n, at [7.917, ..., 7.917]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/alpinen2fcn.html

        @param solution: A numpy array like: [1, 2, 10, 4, ...]
        @return: fx
        """
        return np.prod(np.sqrt(solution)*np.sin(solution))


    def _brown__(self, solution=None):
        """
        Class: uni-modal, convex, differentiable, non-separable
        Global: one global minimum fx = 0 at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/brownfcn.html

        @param solution: A numpy array with x_i in [-1, 4]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += (solution[i]**2)**(solution[i+1]**2+1) + (solution[i+1]**2)**(solution[i]**2+1)
        return result


    def _exponential__(self, solution=None):
        """
        Class: uni-modal, convex, differentiable, non-separable, continuous
        Global: one global minimum fx = 0, at [0,...,0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/exponentialfcn.html

        @param solution: A numpy array with x_i in [-1, 1]
        @return: fx
        """
        return -np.exp(0-.5*np.sum(solution**2))


    def _griewank__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/griewankfcn.html

        @param solution: A numpy array with x_i in [-600, 600]
        @return: fx
        """
        d = len(solution)
        result = 1 + np.sum(solution**2) / 4000
        prod = 1.0
        for i in range(0, d):
            prod *= np.cos(solution[i]/np.sqrt(i+1))
        return result - prod


    def _happy_cat__(self, solution=None, alpha=1.0/8):
        """
        Class: multimodal, non-convex, differentiable, non-separable, parametric
        Global: one global minimum fx = 0, at [-1, ..., -1]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/happycatfcn.html

        @param solution: A numpy array with x_i in [-2, 2]
        @return: fx
        """

        return ((np.sum(solution**2) - len(solution))**2)**alpha + (0.5*np.sum(solution**2)+np.sum(solution))/len(solution) + 0.5


    def _periodic__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, non-separable, continuous
        Global: one global minimum fx = 0.9, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/periodicfcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        return 1 + np.sum(np.sin(solution)**2) - 0.1*np.exp(np.sum(solution**2))


    def _powell_sum__(self, solution=None):
        """
        Class: uni-modal, convex, non-differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/powellsumfcn.html

        @param solution: A numpy array with x_i in [-1, 1]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            result += np.abs(solution[i])**(i+2)
        return result


    def _qing__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, non-separable, continuous
        Global: one global minimum fx = 0, at (±√i,…,±√i)
        Link: http://benchmarkfcns.xyz/benchmarkfcns/qingfcn.html

        @param solution: A numpy array with x_i in [-500, 500]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            result += (solution[i]**2 - i - 1)**2
        return result


    def _quartic__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, separable, continuous, random
        Global: one global minimum fx = 0 + random, at (0, ...,0)
        Link: http://benchmarkfcns.xyz/benchmarkfcns/quarticfcn.html

        @param solution: A numpy array with x_i in [-1.28, 1.28]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            result+= (i+1)*solution[i]**4
        return result+np.random.uniform(0, 1)


    def _rastrigin__(self, solution=None):
        """
        Class: multimodal, convex, differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/rastriginfcn.html

        @param solution: A numpy array with x_in in [-5.12, 5.12]
        @return: fx
        """
        return 10*len(solution) + np.sum(solution**2-10*np.cos(2*solution*np.pi))


    def _ridge__(self, solution=None, d=2, alpha=0.5):
        """
        Class: uni-model, non-convex, differentiable, non-separable
        Global:
        Link: http://benchmarkfcns.xyz/benchmarkfcns/ridgefcn.html

        @param solution: A numpy array with x_i in [-5, 5]
        @return: fx
        """
        t1 = solution[1:]
        return solution[0] + d*np.sum(t1**2)**alpha


    def _rosenbrock__(self, solution=None, a=1, b=100):
        """
        Class: multimodal, non-convex, differentiable, non-separable, continuous
        Global: 1 global optima, fx = 0, x = [1, ..., 1]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/rosenbrockfcn.html

        @param solution: A numpy array with x_i in [-5, 10]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += b*(solution[i+1] - solution[i]**2)**2 + (a-solution[i])**2
        return result


    def _salomon__(self, solution=None):
        """
        Class: multimodal, non-convex, differentiable, non-separable, continuous
        Global: 1 global optima, fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/salomonfcn.html

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return 1 - np.cos(2*np.pi*np.sqrt(np.sum(solution**2))) + 0.1*np.sqrt(np.sum(solution**2))


    def _schwefel_2_20__(self, solution=None):
        """
        Class: uni-modal, convex, non-differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefel220fcn.html

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.sum(np.abs(solution))


    def _schwefel_2_21__(self, solution=None):
        """
        Class: uni-modal, convex, non-differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefel221fcn.html

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.max(np.abs(solution))


    def _schwefel_2_22__(self, solution=None):
        """
        Class: uni-modal, convex, non-differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefel222fcn.html

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.sum(np.abs(solution)) + np.prod(np.abs(solution))


    def _schwefel_2_23__(self, solution=None):
        """
        Class: uni-modal, convex, differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefel221fcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        return np.sum(solution**10)


    def _schwefel__(self, solution=None):
        """
        Class: multi-modal, non-convex, non-differentiable, non-separable, continuous
        Global: one global minimum fx = 0, at [420.9687, ..., 420.9687]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefelfcn.html

        @param solution: A numpy array with x_i in [-500, 500]
        @return: fx
        """
        return 418.9829*len(solution) - np.sum(solution*np.sin(np.sqrt(np.abs(solution))))


    def _shubert_3__(self, solution=None):
        """
        Class: multi-modal, non-convex, differentiable, separable, continuous
        Global: one global minimum fx = -29.6733337
        Link: http://benchmarkfcns.xyz/benchmarkfcns/shubert3fcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            for j in range(1, 6):
                result+= j*np.sin((j+1)*solution[i] + j)
        return result

    def _shubert_4__(self, solution=None):
        """
        Class: multi-modal, non-convex, differentiable, separable, continuous
        Global: one global minimum fx = -25.740858
        Link: http://benchmarkfcns.xyz/benchmarkfcns/shubert4fcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            for j in range(1, 6):
                result += j * np.cos((j + 1) * solution[i] + j)
        return result


    def _shubert__(self, solution=None):
        """
        Class: multi-modal, non-convex, differentiable, non-separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/shubertfcn.html

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        d = len(solution)
        prod = 1.0
        for i in range(0, d):
            result = 0
            for j in range(1, 6):
                result += np.cos((j + 1) * solution[i] + j)
            prod *= result
        return prod


    def _sphere__(self, solution=None):
        """
        Class: uni-modal, convex, differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/spherefcn.html

        @param solution: A numpy array with x_i in [-5.12, 5.12]
        @return: fx
        """
        return np.sum(solution**2)


    def _styblinski__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous
        Global: one global minimum fx = -39.16599 * d , at [-2.903534, ..., -2.903534]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/styblinskitankfcn.html

        @param solution: A numpy array with x_i in [-5, 5]
        @return: fx
        """
        return 0.5*np.sum(solution**4 - 16*solution**2 + 5*solution)


    def _sum_squres__(self, solution=None):
        """
        Class: uni-modal, convex, differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/sumsquaresfcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = 0.0
        for i in range(0, d):
            result = (i+1)*solution[i]**2
        return result


    def _xin_she_yang__(self, solution=None):
        """
        Class: multi-modal, non-convex, non-differentiable, separable, random
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/xinsheyangn1fcn.html

        @param solution: A numpy array with x_i in [-5, 5]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            result += np.random.uniform(0, 1) * np.abs(solution[i])**(i+1)
        return result


    def _xin_she_yang_n2__(self, solution=None):
        """
        Class: multi-modal, non-convex, non-differentiable, non-separable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/xinsheyangn2fcn.html

        @param solution: A numpy array with x_i in [-2pi, 2pi]
        @return: fx
        """
        return np.sum(np.abs(solution))*np.exp(-np.sum(np.sin(solution**2)))


    def _xin_she_yang_n3__(self, solution=None, m=5, beta=15):
        """
        Class: uni-modal, non-convex, differentiable, non-separable, parametric
        Global: one global minimum fx = -1, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/xinsheyangn3fcn.html

        @param solution: A numpy array with x_i in [-2pi, 2pi]
        @return: fx
        """
        t1 = np.exp(-np.sum( np.power(solution/beta, 2*m)))
        t2 = -2*np.exp(-np.sum(solution**2))
        t3 = np.prod(np.cos(solution)**2)
        return t1 + t2*t3


    def _xin_she_yang_n4__(self, solution=None):
        """
        Class: multi-modal, non-convex, non-differentiable, non-separable
        Global: one global minimum fx = -1, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/xinsheyangn4fcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        t1 = np.sum(np.sin(solution)**2)
        t2 = -np.exp(-np.sum(solution**2))
        t3 = -np.exp(np.sum(np.sin(np.sqrt(np.abs(solution)))**2))
        return (t1 + t2) * t3


    def _zakharov__(self, solution=None):
        """
        Class: uni-modal, convex, continuous
        Global: one global minimum fx = -1, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/zakharov.html

        @param solution: A numpy array with x_i in [-5, 10]
        @return: fx
        """
        t1 = np.sum(solution**2)
        t2 = 0
        d = len(solution)
        for i in range(0, d):
            t2 += 0.5*(i+1)*solution[i]
        return t1 + t2**2 + t2**4









#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 20:09, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from opfunu.cec.cec2005.root import Root
from numpy import sum, dot, sin, sqrt, array, cos, pi, exp, ones, max


class Model(Root):
    def __init__(self, f_name="Rotated Hybrid Composition Function 3 with High Condition Number Matrix",
                 f_shift_data_file="data_hybrid_func3",
                 f_ext='.txt', f_bias=360, f_matrix=None):
        Root.__init__(self, f_name, f_shift_data_file, f_ext, f_bias)
        self.f_matrix = f_matrix

    def __f12__(self, solution=None):
        def __xy__(x, y):
            return 0.5 + (sin(sqrt(x ** 2 + y ** 2)) ** 2 - 0.5) / (1 + 0.001 * (x ** 2 + y ** 2)) ** 2

        result = __xy__(solution[-1], solution[0])
        for i in range(0, len(solution) - 1):
            result += __xy__(solution[i], solution[i + 1])
        return result

    def __f34__(self, solution=None):
        return sum(solution ** 2 - 10 * cos(2 * pi * solution) + 10)

    def __f56__(self, solution=None):
        def __f8__(x):
            return x ** 2 / 4000 - cos(x / sqrt(x)) + 1

        def __f2__(x, y):
            return 100 * (x ** 2 - y) ** 2 + (x - 1) ** 2

        result = __f8__(__f2__(solution[-1], solution[0]))
        for i in range(0, len(solution) - 1):
            result += __f8__(__f2__(solution[i], solution[i + 1]))
        return result

    def __f78__(self, solution=None, a=0.5, b=3, k_max=20):
        result = 0.0
        for i in range(len(solution)):
            result += sum([a ** k * cos(2 * pi * b ** k * (solution + 0.5)) for k in range(0, k_max)])
        return result - len(solution) * sum([a ** k * cos(2 * pi * b ** k * 0.5) for k in range(0, k_max)])

    def __f910__(self, solution=None):
        result = sum(solution ** 2) / 4000
        temp = 1.0
        for i in range(len(solution)):
            temp *= cos(solution[i] / sqrt(i + 1))
        return result - temp + 1

    def __fi__(self, solution=None, idx=None):
        if idx == 0 or idx == 1:
            return self.__f12__(solution)
        elif idx == 2 or idx == 3:
            return self.__f34__(solution)
        elif idx == 4 or idx == 5:
            return self.__f56__(solution)
        elif idx == 6 or idx == 7:
            return self.__f78__(solution)
        else:
            return self.__f910__(solution)

    def _main__(self, solution=None):
        problem_size = len(solution)
        if problem_size > 100:
            print("CEC 2005 not support for problem size > 100")
            return 1
        if problem_size == 10 or problem_size == 30 or problem_size == 50:
            self.f_matrix = "hybrid_func3_HM_D" + str(problem_size)
        else:
            print("CEC 2005 F22 function only support problem size 10, 30, 50")
            return 1
        num_funcs = 10
        C = 2000
        xichma = array([1.0, 1.0, 1.0, 1.0, 1.0, 2.0, 2.0, 2.0, 2.0, 2.0])
        lamda = array([5.0 * 5.0 / 100.0, 5.0 / 100.0, 5.0 * 1.0, 1.0, 5.0 * 1.0, 1.0, 5.0 * 10.0, 10.0, 5.0 * 5.0 / 200.0, 5.0 / 200.0])
        bias = array([0, 100, 200, 300, 400, 500, 600, 700, 800, 900])
        y = 5 * ones(problem_size)
        shift_data = self.load_matrix_data(self.f_shift_data_file)
        shift_data = shift_data[:, :problem_size]
        matrix = self.load_matrix_data(self.f_matrix)

        weights = ones(num_funcs)
        fits = ones(num_funcs)
        for i in range(0, num_funcs):
            w_i = exp(-sum((solution - shift_data[i]) ** 2) / (2 * problem_size * xichma[i] ** 2))
            z = dot((solution - shift_data[i]) / lamda[i], matrix[i * problem_size:(i + 1) * problem_size, :])
            fit_i = self.__fi__(z, i)
            f_maxi = self.__fi__(dot((y / lamda[i]), matrix[i * problem_size:(i + 1) * problem_size, :]), i)
            fit_i = C * fit_i / f_maxi

            weights[i] = w_i
            fits[i] = fit_i

        sw = sum(weights)
        maxw = max(weights)

        for i in range(0, num_funcs):
            if weights[i] != maxw:
                weights[i] = weights[i] * (1 - maxw ** 10)
            weights[i] = weights[i] / sw

        fx = sum(dot(weights, (fits + bias)))
        return fx + self.f_bias



#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 15:29, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from opfunu.cec.cec2008.root import Root
from numpy import sum
from numpy.random import seed, choice, uniform


class Model(Root):
    def __init__(self, f_name="FastFractal “DoubleDip” Function", f_shift_data_file=None, f_ext='.txt', f_bias=None, f_matrix=None):
        Root.__init__(self, f_name, f_shift_data_file, f_ext, f_bias)
        self.f_matrix = f_matrix

    def _main__(self, solution=None):
        seed(0)
        problem_size = len(solution)
        if problem_size > 1000:
            print("CEC 2008 not support for problem size > 1000")
            return 1

        def __doubledip__(x, c, s):
            if -0.5 < x < 0.5:
                return (-6144*(x - c)**6 + 3088*(x - c)**4 - 392*(x - c)**2 + 1)*s
            else:
                return 0

        def __fractal1d__(x):
            result1 = 0.0
            for k in range(1, 4):
                result2 = 0.0
                upper = 2**(k-1)
                for t in range(1, upper):
                    selected = choice([0, 1, 2])
                    result2 += sum([ __doubledip__(x, uniform(), 1.0 / (2**(k-1)*(2-uniform()))) for _ in range(0, selected)])
                result1 += result2
            return result1

        def __twist__(y):
            return 4*(y**4 - 2*y**3 + y**2)

        result = solution[-1] + __twist__(solution[0])
        for i in range(0, problem_size-1):
            x = solution[i] + __twist__(solution[i%problem_size + 1])
            result += __fractal1d__(x)
        return result



#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 12:31, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

import pkg_resources
from pandas import read_csv
from numpy import cos, sqrt, pi, e, exp, sum

CURRENT_PATH = pkg_resources.resource_filename("opfunu", "cec/cec2010/")
SUPPORT_PATH_DATA = pkg_resources.resource_filename("opfunu", "cec/cec2010/support_data/")

def load_shift_data__(data_file=None):
    data = read_csv(SUPPORT_PATH_DATA + data_file, delimiter='\s+', index_col=False, header=None)
    return data.values.reshape((-1))

def load_matrix_data__(data_file=None):
    data = read_csv(SUPPORT_PATH_DATA + data_file, delimiter='\s+', index_col=False, header=None)
    return data.values

def f1_sphere__(solution=None):
    return sum(solution**2)

def f2_elliptic__(solution=None):
    result = 0.0
    for i in range(0, len(solution)):
        result += (10**6)**(i/(len(solution)-1)) * solution[i]**2
    return result

def f3_rastrigin__(solution=None):
    return sum(solution ** 2 - 10 * cos(2 * pi * solution) + 10)

def f4_ackley__(solution=None):
    return -20 * exp(-0.2 * sqrt(sum(solution ** 2) / len(solution))) - exp(sum(cos(2 * pi * solution)) / len(solution)) + 20 + e

def f5_schwefel__(solution=None):
    return sum([ sum(solution[:i])**2 for i in range(0, len(solution))])

def f6_rosenbrock__(solution=None):
    result = 0.0
    for i in range(len(solution) - 1):
        result += 100 * (solution[i] ** 2 - solution[i + 1]) ** 2 + (solution[i] - 1) ** 2
    return result


#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 12:31, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

import pkg_resources
from pandas import read_csv


class Root:
    def __init__(self, f_name=None, f_shift_data_file=None, f_ext='.txt', f_bias=None):
        self.f_name = f_name
        self.f_shift_data_file = f_shift_data_file
        self.f_ext = f_ext
        self.f_bias = f_bias
        self.current_path = pkg_resources.resource_filename("opfunu", "cec/cec2008/")
        self.support_path_data = pkg_resources.resource_filename("opfunu", "cec/cec2008/support_data/")

    def load_shift_data(self):
        data = read_csv(self.support_path_data + self.f_shift_data_file + self.f_ext, delimiter='\s+', index_col=False, header=None)
        return data.values.reshape((-1))

    def load_matrix_data(self, data_file=None):
        data = read_csv(self.support_path_data + data_file + self.f_ext, delimiter='\s+', index_col=False, header=None)
        return data.values




#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 10:11, 16/03/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from opfunu.cec.cec2013.unconstraint import Model as ObjFunc13
from opfunu.cec.cec2014.unconstraint import Model as ObjFunc14
from mealpy.evolutionary_based.GA import BaseGA
from mealpy.swarm_based.WOA import BaseWOA
from mealpy.human_based.TLO import BaseTLO
from mealpy.physics_based.HGSO import BaseHGSO, LevyHGSO

from opfunu.cec_basic.cec2014 import *

## Setting parameters
problem_size = 30
func = ObjFunc14(problem_size)
domain_range = [-15, 15]
log = True
epoch = 500
pop_size = 50

# pc = 0.95
# pm = 0.025
# md = BaseGA(func.F1, problem_size, domain_range, log, epoch, pop_size, pc, pm)
# best_position, best_fit, list_loss = md._train__()
# print(best_fit)

md2 = BaseWOA(F18, problem_size, domain_range, log, epoch, pop_size)
best_position2, best_fit2, list_loss2 = md2._train__()
print(best_fit2)

md3 = BaseTLO(F18, problem_size, domain_range, log, epoch, pop_size)
best_position3, best_fit3, list_loss3 = md3._train__()
print(best_fit3)

md4 = BaseHGSO(F18, problem_size, domain_range, log, epoch, pop_size)
best_position4, best_fit4, list_loss4 = md4._train__()
print(best_fit4)

md5 = LevyHGSO(F18, problem_size, domain_range, log, epoch, pop_size)
best_position5, best_fit5, list_loss5 = md5._train__()
print(best_fit5)


#!/usr/bin/env python                                                                                   #
# ------------------------------------------------------------------------------------------------------#
# Created by "Thieu Nguyen" at 16:32, 07/12/2019                                                        #
#                                                                                                       #
#       Email:      nguyenthieu2102@gmail.com                                                           #
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  #
#       Github:     https://github.com/thieunguyen5991                                                  #
#-------------------------------------------------------------------------------------------------------#

import numpy as np

class Functions:
    """
        This class of functions is belongs to multi-modal function, all functions will be scaled to n-dimension space
    """

    def _ackley__(self, solution=None):
        """
        Class: Continuous, Non-convex, Differentiable
        Global: one global minimum fx = 0, at [0, 0,...0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/ackleyfcn.html

        @param solution: A numpy array like with x_i in [-32, 32]
        @return: fx
        """
        a, b, c = 20, 0.2, 2*np.pi
        d = len(solution)
        sum_1 = -a*np.exp(-b* np.sqrt(np.sum(solution ** 2) / d))
        sum_2 = np.exp(np.sum(np.cos(c*solution))/d)
        return sum_1 - sum_2 + a + np.exp(1)


    def _ackley_4__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: −4.590101633799122
        Link: http://benchmarkfcns.xyz/benchmarkfcns/ackleyn4fcn.html

        @param solution: A numpy array like with x_i in [-35, 35]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += np.exp(-0.2*np.sqrt(solution[i]**2 + solution[i+1]**2)) + 3*(np.cos(2*solution[i]) + np.sin(2*solution[i+1]))
        return result


    def _alpine_1__(self, solution=None):
        """
        Class: Continuous, Non-Differentiable, Separable, Non-Scalable
        Global: 1 global minimum, fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/alpinen1fcn.html

        @param solution: A numpy array like with x_i in [0, 10]
        @return: fx
        """
        return np.sum(np.dot(solution, np.sin(solution)) + 0.1 * solution)


    def _alpine_2__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Scalable
        Global: 1 global minimum, fx = 2.808^D, at [7.917, ..., 7.917]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/alpinen2fcn.html

        @param solution: A numpy array like with x_i in [0, 10]
        @return: fx
        """
        return np.prod(np.sqrt(solution)*np.sin(solution))


    def _cosine_mixture__(self, solution=None):
        """
        Class: Discontinuous, Non-Differentiable, Separable, Scalable
        Global:
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array like with x_i in [-1, 1]
        @return: fx
        """
        return -0.1*np.sum(np.cos(5*np.pi*solution)) - np.sum(solution**2)


    def _csendes__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Scalable
        Global: fx = 0, at [0, ...,0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array like with x_i in [-1, 1]
        @return: fx
        """
        return np.sum(solution**6*(2+np.sin(1.0/solution)))


    def _deb_1__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Scalable
        Global:
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array like with x_i in [-1, 1]
        @return: fx
        """
        return -np.sum(np.sin(5*np.pi*solution)**6) / len(solution)


    def _deb_3__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Scalable
        Global:
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array like with x_i in [-1, 1]
        @return: fx
        """
        return -np.sum(np.sin(5*np.pi*(solution**0.75 - 0.05))**6) / len(solution)


    def _egg_holder__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: 959.64
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array like with x_i in [-512, 512]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += -(solution[i+1]+47) * np.sin(np.sqrt(np.abs(solution[i+1] + solution[i] / 2 + 47))) -\
                solution[i]*np.sin(np.sqrt(np.abs(solution[i] - solution[i+1] - 47)))
        return result


    def _exponential__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: one global minimum fx = 1, at [0,...,0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-1, 1]
        @return: fx
        """
        return -np.exp(-0.5*np.sum(solution**2))


    def _griewank__(self, solution=None):
        """
        Class: uni-modal, non-convex, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/griewankfcn.html

        @param solution: A numpy array with x_i in [-600, 600]
        @return: fx
        """
        d = len(solution)
        result = 1 + np.sum(solution**2) / 4000
        prod = 1.0
        for i in range(0, d):
            prod *= np.cos(solution[i]/np.sqrt(i+1))
        return result - prod


    def _mishra_1__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: one global minimum fx = 2
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [0, 1]
        @return: fx
        """
        d = len(solution)
        result = d - np.sum(solution[:d-1])
        return (1+result)**result


    def _mishra_2__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: one global minimum fx = 2
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [0, 1]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += 0.5*(solution[i]+solution[i+1])
        result = d - result
        return (1+result)**result


    def _mishra_7__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Non-Scalable
        Global: one global minimum fx = 0
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array
        @return: fx
        """
        return (np.prod(solution) - np.math.factorial(len(solution)))**2


    def _mishra_11__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Non-Scalable
        Global: one global minimum fx = 0
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array
        @return: fx
        """
        d = len(solution)
        return (np.sum(np.abs(solution))/d - (np.prod(np.abs(solution)))**(1/d))**2


    def _pathological__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Non-Scalable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += 0.5 + ( np.sin(np.sqrt(100*solution[i]**2 + solution[i+1]**2))**2 -0.5 ) / \
                (1 + 0.001*(solution[i]**2 - 2*solution[i]*solution[i+1] + solution[i+1]**2)**2)
        return result


    def _pinter__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-separable, Scalable
        Global: global minimum fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result1 = 0
        result2 = 0
        result3 = 0
        for i in range(0, d):
            result1 += (i+1)*solution[i]**2
            if i==0:
                result2 += 20*(i+1)*np.sin(np.sin(solution[i])*solution[d-1] + np.sin(solution[i+1]))**2
                result3 += (i+1)*np.log10(1+(i+1)* (solution[d-1]**2 - 2*solution[i]+3*solution[i+1]-np.cos(solution[i])+1)**2 )
            if i==d-1:
                result2 += 20 * (i + 1) * np.sin(np.sin(solution[i]) * solution[i - 1] + np.sin(solution[0])) ** 2
                result3 += (i + 1) * np.log10(1 + (i + 1) * (solution[i - 1] ** 2 - 2 * solution[i] + 3 * solution[0] - np.cos(solution[i]) + 1) ** 2)
            result2 += 20*(i+1)*np.sin(solution[i-1]*np.sin(solution[i]))**2
            result3 += (i+1)*np.log10(1 + (i+1) * (solution[i-1]**2-2*solution[i]+3*solution[i+1]-np.cos(solution) + 1)**2 )
        return result1 + result2 + result3


    def _qing__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable Scalable
        Global: one global minimum fx = 0, at (±√i,…,±√i)
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-500, 500]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            result += (solution[i]**2 - i - 1)**2
        return result


    def _quintic__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Non-Scalable
        Global: one global minimum fx = 0
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        return np.sum(np.abs(solution**5-3*solution**4+4*solution**3+2*solution**2-10*solution-4))


    def _rana__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: one global minimum fx = 0
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-500, 500]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            t1 = np.sqrt(np.abs(solution[i+1] + solution[i] + 1))
            t2 = np.sqrt(np.abs(solution[i+1] - solution[i] + 1))
            result += (solution[i+1]+1)*np.cos(t2)*np.sin(t1) + solution[i]*np.cos(t1)*np.sin(t2)
        return result


    def _salomon__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: 1 global optima, fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return 1 - np.cos(2*np.pi*np.sqrt(np.sum(solution**2))) + 0.1*np.sqrt(np.sum(solution**2))


    def _schwefel_2_4__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Non-Scalable
        Global: one global minimum fx = 0, at [1, ..., 1]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [0, 10]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            result += (solution[i]-1)**2 + (solution[0] - solution[i]**2)**2
        return result


    def _schwefel_2_25__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Non-Scalable
        Global: one global minimum fx = 0, at [1, ..., 1]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [0, 10]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(1, d):
            result += (solution[i]-1)**2 + (solution[0] - solution[i]**2)**2
        return result


    def _schwefel_2_26__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Scalable
        Global: one global minimum fx = -418.983
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-500, 500]
        @return: fx
        """
        return -np.sum(solution*np.sin(np.sqrt(np.abs(solution)))) / len(solution)


    def _shubert__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Non-Scalable
        Global: one global minimum fx = -186.7309
        Link: http://benchmarkfcns.xyz/benchmarkfcns/shubertfcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = 1
        for i in range(0, d):
            temp = 0
            for j in range(1, 6):
                temp += np.cos(solution[i]*(j+1) + j)
            result *= temp
        return result


    def _shubert_3__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Non-Scalable
        Global: one global minimum fx = -29.6733337
        Link: http://benchmarkfcns.xyz/benchmarkfcns/shubert3fcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = 0.0
        for i in range(0, d):
            temp = 0
            for j in range(1, 6):
                temp += j*np.sin(solution[i]*(j+1) + j)
            result += temp
        return result


    def _shubert_4__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Non-Scalable
        Global: one global minimum fx = -25.740858
        Link: http://benchmarkfcns.xyz/benchmarkfcns/shubert4fcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = 0.0
        for i in range(0, d):
            temp = 0
            for j in range(1, 6):
                temp += j*np.cos(solution[i]*(j+1) + j)
            result += temp
        return result


    def _schaffer_f6__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += 0.5 + (np.sin(np.sqrt(solution[i]**2+solution[i+1]**2))**2 -0.5) / \
                      (1 + 0.001*(solution[i]**2 + solution[i+1]**2))**2
        return result


    def _styblinski_tang_(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Non-Scalable
        Global: one global minimum fx = -78.332
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-5, 5]
        @return: fx
        """
        return 0.5*np.sum(solution**4 - 16*solution**2 + 5*solution)


    def _trid_6__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Non-Scalable
        Global: one global minimum fx = -50
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-36, 36]
        @return: fx
        """
        result = np.sum((solution - 1) ** 2)
        for i in range(1, len(solution)):
            result -= solution[i] * solution[i-1]
        return result


    def _trigonometric_1__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [0, pi]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            result += ( d - np.sum(np.cos(solution)) + (i+1)*(1 - np.cos(solution[i]) - np.sin(solution[i])) )**2
        return result


    def _trigonometric_2__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: one global minimum fx = 1, at [0.9, ..., 0.9]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-500, 500]
        @return: fx
        """
        d = len(solution)
        result = 1
        for i in range(0, d):
            result += 8 * np.sin(7*(solution[i] - 0.9)**2) + 6 * np.sin(14*(solution[0]-0.9)**2) + (solution[i]-0.9)**2
        return result




#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 11:16, 26/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from numpy import dot, array, sum, matmul, where, sqrt, sign, min, cos, pi, exp, round
from opfunu.cec.utils import BasicFunction


class Model(BasicFunction):
    def __init__(self, solution=None, cec_type="cec2013", f_shift="shift_data", f_matrix="M_D", bound=(-100, 100),
                 dimensions=(2, 5, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100)):
        BasicFunction.__init__(self, cec_type)
        self.problem_size = len(solution)
        self.dimensions = dimensions
        self.check_dimensions(self.problem_size)
        self.bound = bound
        self.solution = solution
        self.f_shift = f_shift + ".txt"
        self.f_matrix = f_matrix + str(self.problem_size) + ".txt"
        self.shift = self.load_matrix_data__(self.f_shift)
        self.matrix = self.load_matrix_data__(self.f_matrix)

    def F1(self, name="Sphere Function", f_bias=-1400):
        return sum((self.solution - self.shift[3:4, :self.problem_size]) ** 2) + f_bias

    def F2(self, name="Rotated High Conditioned Elliptic Function", f_bias=-1300):
        t1 = self.solution - self.shift[4:5, :self.problem_size]
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = self.osz_func__(t2)
        t4 = self.elliptic__(t3)
        return t4 + f_bias

    def F3(self, name="Rotated Bent Cigar Function", f_bias=-1200):
        t1 = self.solution - self.shift[2:3, :self.problem_size]
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = self.asy_func__(t2, beta=0.5)
        t4 = dot(self.matrix[self.problem_size:2 * self.problem_size, :], t3)
        return self.bent_cigar__(t4) + f_bias

    def F4(self, name="Rotated Discus Function", f_bias=-1100):
        t1 = self.solution - self.shift[8:9, :self.problem_size]
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = self.osz_func__(t2)
        return self.discus__(t3) + f_bias

    def F5(self, name="Different Powers Function", f_bias=-1000):
        t1 = self.solution - self.shift[1:2, :self.problem_size]
        return self.different_powers__(t1) + f_bias

    def F6(self, name="Rotated Rosenbrock’s Function", f_bias=-900):
        t1 = 2.048 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = dot(self.matrix[:self.problem_size, :], t1.T) + 1
        return self.rosenbrock__(t2) + f_bias

    def F7(self, name="Rotated Schaffers F7 Function", f_bias=-800):
        t1 = self.solution - self.shift[2:3, :self.problem_size]
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = self.asy_func__(t2, 0.5)
        t4 = self.create_diagonal_matrix__(self.problem_size, alpha=10)
        t5 = matmul(t4, self.matrix[self.problem_size: 2 * self.problem_size, :])
        t6 = dot(t5, t3)
        return self.schaffers_f7__(t6) + f_bias

    def F8(self, name="Rotated Ackley’s Function", f_bias=-700):
        t1 = self.solution - self.shift[:1, :self.problem_size]
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = self.asy_func__(t2, 0.5)
        t4 = self.create_diagonal_matrix__(self.problem_size, alpha=10)
        t5 = matmul(t4, self.matrix[self.problem_size: 2 * self.problem_size, :])
        t6 = dot(t5, t3)
        return self.ackley__(t6) + f_bias

    def F9(self, name="Rotated Weierstrass Function", f_bias=-600):
        t1 = 0.5 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = self.asy_func__(t2, 0.5)
        t4 = self.create_diagonal_matrix__(self.problem_size, alpha=10)
        t5 = matmul(t4, self.matrix[self.problem_size: 2 * self.problem_size, :])
        t6 = dot(t5, t3)
        return self.weierstrass__(t6) + f_bias

    def F10(self, name="Rotated Griewank’s Function", f_bias=-500):
        t1 = 600 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = self.create_diagonal_matrix__(self.problem_size, alpha=100)
        t3 = matmul(t2, self.matrix[:self.problem_size, :])
        t4 = dot(t3, t1.T)
        return self.griewank__(t4) + f_bias

    def F11(self, name="Rastrigin’s Function", f_bias=-400):
        t1 = 5.12 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = self.osz_func__(t1)
        t3 = self.asy_func__(t2, beta=0.2)
        t4 = self.create_diagonal_matrix__(self.problem_size, alpha=10)
        t5 = dot(t4, t3)
        return self.rastrigin__(t5) + f_bias

    def F12(self, name="Rotated Rastrigin’s Function", f_bias=-300):
        t1 = 5.12 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = self.osz_func__(t2)
        t4 = self.asy_func__(t3, beta=0.2)
        t5 = self.create_diagonal_matrix__(self.problem_size, alpha=10)
        t6 = matmul(self.matrix[:self.problem_size, :], t5)
        t7 = matmul(t6, self.matrix[self.problem_size: 2 * self.problem_size, :])
        t8 = dot(t7, t4)
        return self.rastrigin__(t8) + f_bias

    def F13(self, name="Non-continuous Rotated Rastrigin’s Function", f_bias=-200):
        t1 = 5.12 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = where(abs(t2) > 0.5, round(2 * t2) / 2, t2)
        t4 = self.osz_func__(t3)
        t5 = self.asy_func__(t4, beta=0.2)
        t6 = self.create_diagonal_matrix__(self.problem_size, alpha=10)
        t7 = matmul(self.matrix[:self.problem_size, :], t6)
        t8 = matmul(t7, self.matrix[self.problem_size: 2 * self.problem_size, :])
        t9 = dot(t8, t5)
        return self.rastrigin__(t9) + f_bias

    def F14(self, name="Schwefel’s Function", f_bias=-100):
        t1 = 1000 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = self.create_diagonal_matrix__(self.problem_size, alpha=10)
        t3 = dot(t2, t1.T) + 4.209687462275036e+002
        return self.modified_schwefel__(t3) + f_bias

    def F15(self, name="Rotated Schwefel’s Function", f_bias=100):
        t1 = 1000 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = self.create_diagonal_matrix__(self.problem_size, alpha=10)
        t3 = matmul(t2, self.matrix[:self.problem_size, :])
        t4 = dot(t3, t1.T) + 4.209687462275036e+002
        return self.modified_schwefel__(t4) + f_bias

    def F16(self, name="Rotated Katsuura Function", f_bias=200):
        t1 = 5 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)
        t3 = self.create_diagonal_matrix__(self.problem_size, alpha=100)
        t4 = matmul(self.matrix[self.problem_size:2 * self.problem_size, :], t3)
        t5 = dot(t4, t2)
        return self.katsuura__(t5) + f_bias

    def F17(self, name="Lunacek bi-Rastrigin Function", f_bias=300):
        d = 1
        s = 1 - 1.0 / (2 * sqrt(self.problem_size + 20) - 8.2)
        miu0 = 2.5
        miu1 = -sqrt((miu0 ** 2 - d) / s)
        t1 = 10 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = 2 * sign(self.solution) * t1 + miu0
        t3 = self.create_diagonal_matrix__(self.problem_size, alpha=100)
        t4 = dot(t3, (t2 - miu0).T)
        vp1 = sum((t2 - miu0) ** 2)
        vp2 = d * self.problem_size + s * sum((t2 - miu1) ** 2) + 10 * (self.problem_size - sum(cos(2 * pi * t4)))
        return min([vp1, vp2]) + f_bias

    def F18(self, name="Rotated Lunacek bi-Rastrigin Function", f_bias=400):
        d = 1
        s = 1 - 1.0 / (2 * sqrt(self.problem_size + 20) - 8.2)
        miu0 = 2.5
        miu1 = -sqrt((miu0 ** 2 - d) / s)
        t1 = 10 * (self.solution - self.shift[:1, :self.problem_size]) / 100
        t2 = 2 * sign(t1) * t1 + miu0
        t3 = self.create_diagonal_matrix__(self.problem_size, alpha=100)
        t4 = matmul(self.matrix[self.problem_size:2 * self.problem_size, :], t3)
        t5 = dot(self.matrix[:self.problem_size, :], (t2 - miu0).T)
        t6 = dot(t4, t5)
        vp1 = sum((t2 - miu0) ** 2)
        vp2 = d * self.problem_size + s * sum((t2 - miu1) ** 2) + 10 * (self.problem_size - sum(cos(2 * pi * t6)))
        return min([vp1, vp2]) + f_bias

    def F19(self, name="Rotated Expanded Griewank’s plus Rosenbrock’s Function", f_bias=500):
        t1 = 5 * (self.solution - self.shift[4:5, :self.problem_size]) / 100
        t2 = dot(self.matrix[:self.problem_size, :], t1.T) + 1
        return self.expanded_griewank__(t2) + f_bias

    def F20(self, name="Rotated Expanded Scaffer’s F6 Function", f_bias=600):
        t1 = dot(self.matrix[:self.problem_size, :], (self.solution - self.shift[:1, :self.problem_size]).T)
        t2 = self.asy_func__(t1, beta=0.5)
        t3 = dot(self.matrix[self.problem_size: 2 * self.problem_size, :], t2)
        return self.expanded_scaffer__(t3) + f_bias

    def F21(self, name="Composition Function 1", f_bias=700):
        xichma = array([10, 20, 30, 40, 50])
        lamda = array([1, 1e-6, 1e-26, 1e-6, 0.1])
        bias = array([0, 100, 200, 300, 400])
        t1 = self.solution - self.shift[:1, :self.problem_size]
        t2 = dot(self.matrix[:self.problem_size, :], t1.T)

        # g1: Rotated Rosenbrock’s Function f6’
        g1 = lamda[0] * self.F6(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # g2: Rotated Different Powers Function f5’
        g2 = lamda[1] * self.different_powers__(t2) + bias[1]
        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # g3 Rotated Bent Cigar Function f3’
        g3 = lamda[2] * self.F3(f_bias=0) + bias[2]
        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        # g4: Rotated Discus Function f4’
        g4 = lamda[3] * self.F4(f_bias=0) + bias[3]
        w4 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[3] ** 2))

        # g5: Sphere Function f1
        g5 = lamda[4] * self.F1(f_bias=0) + bias[4]
        w5 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[4] ** 2))

        sw = sum([w1, w2, w3, w4, w5])
        result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
        return result + f_bias

    def F22(self, name="Composition Function 2", f_bias=800):
        xichma = array([20, 20, 20])
        lamda = array([1, 1, 1])
        bias = array([0, 100, 200])
        t1 = self.solution - self.shift[:1, :self.problem_size]

        # g1-3: Schwefel's Function f14’
        g1 = lamda[0] * self.F14(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        g2 = lamda[1] * self.F14(f_bias=0) + bias[1]
        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        g3 = lamda[2] * self.F14(f_bias=0) + bias[2]
        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        sw = sum([w1, w2, w3])
        result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
        return result + f_bias

    def F23(self, name="Composition Function 3", f_bias=900):
        xichma = array([20, 20, 20])
        lamda = array([1, 1, 1])
        bias = array([0, 100, 200])
        t1 = self.solution - self.shift[:1, :self.problem_size]

        # g1-3: Schwefel's Function f15’
        g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        g2 = lamda[1] * self.F15(f_bias=0) + bias[1]
        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        g3 = lamda[2] * self.F15(f_bias=0) + bias[2]
        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        sw = sum([w1, w2, w3])
        result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
        return result + f_bias

    def F24(self, name="Composition Function 4", f_bias=1000):
        xichma = array([20, 20, 20])
        lamda = array([0.25, 1, 2.5])
        bias = array([0, 100, 200])
        t1 = self.solution - self.shift[:1, :self.problem_size]

        # g1-3: Schwefel's Function f15’, f12', f9'
        g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        g2 = lamda[1] * self.F12(f_bias=0) + bias[1]
        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        g3 = lamda[2] * self.F9(f_bias=0) + bias[2]
        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        sw = sum([w1, w2, w3])
        result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
        return result + f_bias

    def F25(self, name="Composition Function 5", f_bias=1100):
        xichma = array([10, 30, 50])
        lamda = array([0.25, 1, 2.5])
        bias = array([0, 100, 200])
        t1 = self.solution - self.shift[:1, :self.problem_size]

        # g1-3: Schwefel's Function f15’, f12', f9'
        g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        g2 = lamda[1] * self.F12(f_bias=0) + bias[1]
        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        g3 = lamda[2] * self.F9(f_bias=0) + bias[2]
        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        sw = sum([w1, w2, w3])
        result = (w1 * g1 + w2 * g2 + w3 * g3) / sw
        return result + f_bias

    def F26(self, name="Composition Function 6", f_bias=1200):
        xichma = array([10, 10, 10, 10, 10])
        lamda = array([0.25, 1, 1e-7, 2.5, 10])
        bias = array([0, 100, 200, 300, 400])
        t1 = self.solution - self.shift[:1, :self.problem_size]

        # g1: Rotated Schwefel's Function f15’
        g1 = lamda[0] * self.F15(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # g2: Rotated Rastrigin’s Function f12’
        g2 = lamda[1] * self.F12(f_bias=0) + bias[1]
        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # g3: Rotated High Conditioned Elliptic Function f2’
        g3 = lamda[2] * self.F2(f_bias=0) + bias[2]
        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        # g4: Rotated Weierstrass Function f9’
        g4 = lamda[3] * self.F9(f_bias=0) + bias[3]
        w4 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[3] ** 2))

        # g5: Rotated Griewank’s Function f10
        g5 = lamda[4] * self.F10(f_bias=0) + bias[4]
        w5 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[4] ** 2))

        sw = sum([w1, w2, w3, w4, w5])
        result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
        return result + f_bias

    def F27(self, name="Composition Function 7", f_bias=1300):
        xichma = array([10, 10, 10, 20, 20])
        lamda = array([100, 10, 2.5, 2.5, 0.1])
        bias = array([0, 100, 200, 300, 400])
        t1 = self.solution - self.shift[:1, :self.problem_size]

        # g1: Rotated Schwefel's Function f15’
        g1 = lamda[0] * self.F10(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # g2: Rotated Rastrigin’s Function f12’
        g2 = lamda[1] * self.F12(f_bias=0) + bias[1]
        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # g3: Rotated High Conditioned Elliptic Function f2’
        g3 = lamda[2] * self.F15(f_bias=0) + bias[2]
        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        # g4: Rotated Weierstrass Function f9’
        g4 = lamda[3] * self.F9(f_bias=0) + bias[3]
        w4 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[3] ** 2))

        # g5: Rotated Griewank’s Function f10
        g5 = lamda[4] * self.F1(f_bias=0) + bias[4]
        w5 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[4] ** 2))

        sw = sum([w1, w2, w3, w4, w5])
        result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
        return result + f_bias

    def F28(self, name="Composition Function 8", f_bias=1400):
        xichma = array([10, 20, 30, 40, 50])
        lamda = array([2.5, 2.5e-3, 2.5, 2.5e-4, 0.1])
        bias = array([0, 100, 200, 300, 400])
        t1 = self.solution - self.shift[:1, :self.problem_size]

        # g1: Rotated Schwefel's Function f15’
        g1 = lamda[0] * self.F19(f_bias=0) + bias[0]
        w1 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[0] ** 2))

        # g2: Rotated Rastrigin’s Function f12’
        g2 = lamda[1] * self.F7(f_bias=0) + bias[1]
        w2 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[1] ** 2))

        # g3: Rotated High Conditioned Elliptic Function f2’
        g3 = lamda[2] * self.F15(f_bias=0) + bias[2]
        w3 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[2] ** 2))

        # g4: Rotated Weierstrass Function f9’
        g4 = lamda[3] * self.F20(f_bias=0) + bias[3]
        w4 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[3] ** 2))

        # g5: Rotated Griewank’s Function f10
        g5 = lamda[4] * self.F1(f_bias=0) + bias[4]
        w5 = (1.0 / sqrt(sum(t1 ** 2))) * exp(-sum(t1 ** 2) / (2 * self.problem_size * xichma[4] ** 2))

        sw = sum([w1, w2, w3, w4, w5])
        result = (w1 * g1 + w2 * g2 + w3 * g3 + w4 * g4 + w5 * g5) / sw
        return result + f_bias



from setuptools import setup, find_packages

def readme():
    with open('README.md') as f:
        README = f.read()
    return README

setup(
    name="opfunu",
    version="0.6.5",
    author="Thieu Nguyen",
    author_email="nguyenthieu2102@gmail.com",
    description="A framework of un-constrained Optimization Functions in Numpy (OpFuNu) for global optimization problems",
    long_description=readme(),
    long_description_content_type="text/markdown",
    url="https://github.com/thieunguyen5991/opfunu",
    download_url="https://github.com/thieunguyen5991/opfunu/archive/v0.6.5.zip",
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Mathematics",
        "Topic :: System :: Benchmark",
        "License :: OSI Approved :: Apache Software License",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Intended Audience :: Science/Research"
    ],
    install_requires=["numpy"],
    python_requires='>=3.7',
)


#!/usr/bin/env python                                                                                   #
# ------------------------------------------------------------------------------------------------------#
# Created by "Thieu Nguyen" at 02:52, 07/12/2019                                                        #
#                                                                                                       #
#       Email:      nguyenthieu2102@gmail.com                                                           #
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  #
#       Github:     https://github.com/thieunguyen5991                                                  #
#-------------------------------------------------------------------------------------------------------#


import numpy as np

class Functions:
    """
        This class of functions is belongs to 3-dimensional space
    """

    def _wolfe__(self, solution=None):
        """
        Class: multi-modal, non-convex, continuous, differentiable, non-separable
        Global: 1 global minimum fx = 0, [0, 0, 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/wolfefcn.html

        @param solution: A numpy array include 3 items like: [0.2, 0.22, 0.5], limited range: [0, 2]
        """
        n = len(solution)
        assert (n == 3, 'Wolfe function is only defined on a 3D space.')
        return 4/3 * (solution[0]**2 + solution[1]**2 - solution[0]*solution[1])**0.75 + solution[2]



#!/usr/bin/env python                                                                                   #
# ------------------------------------------------------------------------------------------------------#
# Created by "Thieu Nguyen" at 16:32, 07/12/2019                                                        #
#                                                                                                       #
#       Email:      nguyenthieu2102@gmail.com                                                           #
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  #
#       Github:     https://github.com/thieunguyen5991                                                  #
#-------------------------------------------------------------------------------------------------------#

import numpy as np

class Functions:
    """
        This class of functions is belongs to uni-modal function, all functions will be scaled to n-dimension space
    """

    def _brown__(self, solution=None):
        """
        Class: uni-modal, convex, differentiable, non-separable
        Global: one global minimum fx = 0 at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/brownfcn.html

        @param solution: A numpy array with x_i in [-1, 4]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d - 1):
            result += (solution[i] ** 2) ** (solution[i + 1] ** 2 + 1) + (solution[i + 1] ** 2) ** (solution[i] ** 2 + 1)
        return result


    def _chung_reynolds__(self, solution=None):
        """
        Class: continuous, differentiable, partially-separable, scalable
        Global: one global minimum fx = 0 at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.sum(solution**2)**2


    def _dixon_price__(self, solution=None):
        """
        Class: continuous, differentiable, non-separable, scalable
        Global: one global minimum fx = 0
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = (solution[0] - 1)**2
        for i in range(1, d-1):
            result += (i+1)*(2*solution[i]**2 - solution[i-1])**2
        return result


    def _powell_sum__(self, solution=None):
        """
        Class: continuous, differentiable, separable, scalable
        Global: one global minimum fx = 0
        Link: http://benchmarkfcns.xyz/benchmarkfcns/powellsumfcn.html

        @param solution: A numpy array with x_i in [-1, 1]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d):
            result += np.abs(solution[i]) ** (i + 2)
        return result


    def _powell_singular_2__(self, solution=None):
        """
        Class: continuous, differentiable, non-separable, scalable
        Global: one global minimum fx = 0
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-4, 5]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(1, d-2):
            result += (solution[i-1]+10*solution[i])**2 + 5*(solution[i+1]-solution[i+2])**2 + \
                   (solution[i]-2*solution[i+1])**4 + 10 * (solution[i-1]-solution[i+2])**4
        return result


    def _rosenbrock__(self, solution=None):
        """
        Class: non-convex, differentiable, non-separable, continuous
        Global: 1 global optima, fx = 0, x = [1, ..., 1]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-30, 30]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += 100*(solution[i+1] - solution[i]**2)**2 + (solution[i]-1)**2
        return result


    def _schwefel_2_20__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: 1 global optima, fx = 0, x = [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefel220fcn.html

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.sum(np.abs(solution))


    def _schwefel_2_21__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: 1 global optima, fx = 0, x = [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefel221fcn.html

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.max(np.abs(solution))


    def _schwefel_2_22__(self, solution=None):
        """
        Class: convex, non-differentiable, separable, continuous
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefel222fcn.html

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.sum(np.abs(solution)) + np.prod(np.abs(solution))


    def _schwefel_2_23__(self, solution=None):
        """
        Class: differentiable, non-separable, continuous, scalable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/schwefel223fcn.html

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        return np.sum(solution**10)


    def _step__(self, solution=None):
        """
        Class: Discontinuous, Non-Differentiable, Separable, Scalable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.sum(np.floor(np.abs(solution)))


    def _step_2__(self, solution=None):
        """
        Class: Discontinuous, Non-Differentiable, Separable, Scalable
        Global: one global minimum fx = 0, at [-0.5, ..., -0.5]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.sum(np.floor(solution+0.5)**2)


    def _step_3__(self, solution=None):
        """
        Class: Discontinuous, Non-Differentiable, Separable, Scalable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-100, 100]
        @return: fx
        """
        return np.sum(np.floor(solution**2))


    def _streched_v_sin_wave__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = 0
        for i in range(0, d-1):
            result += (solution[i+1]**2 + solution[i]**2)**0.25 * (np.sin(50*(solution[i+1]**2+solution[i]**2)**0.1)**2+0.1)
        return result


    def _sum_squres__(self, solution=None):
        """
        Class: Continuous, Differentiable, Separable, Scalable
        Global: one global minimum fx = 0, at [0, ..., 0]
        Link: A Literature Survey of Benchmark Functions For Global Optimization Problems (2013)

        @param solution: A numpy array with x_i in [-10, 10]
        @return: fx
        """
        d = len(solution)
        result = 0.0
        for i in range(0, d):
            result = (i+1)*solution[i]**2
        return result


    def _zakharov__(self, solution=None):
        """
        Class: Continuous, Differentiable, Non-Separable, Scalable
        Global: fx = 0, at [0, ..., 0]
        Link: http://benchmarkfcns.xyz/benchmarkfcns/zakharov.html

        @param solution: A numpy array with x_i in [-5, 10]
        @return: fx
        """
        d = len(solution)
        result1 = np.sum(solution ** 2)
        result2 = 0
        for i in range(0, d):
            result2 += (i + 1) * solution[i]
        return result1 + (0.5 * result2) ** 2 + (0.5 * result2) ** 4



#!/usr/bin/env python
# ------------------------------------------------------------------------------------------------------%
# Created by "Thieu Nguyen" at 15:51, 20/04/2020                                                        %
#                                                                                                       %
#       Email:      nguyenthieu2102@gmail.com                                                           %
#       Homepage:   https://www.researchgate.net/profile/Thieu_Nguyen6                                  %
#       Github:     https://github.com/thieunguyen5991                                                  %
#-------------------------------------------------------------------------------------------------------%

from opfunu.cec.cec2005.root import Root
from numpy import dot, cos, pi


class Model(Root):
    def __init__(self, f_name="Shifted Rotated Rastrigin's Function", f_shift_data_file="data_rastrigin",
                 f_ext='.txt', f_bias=-330, f_matrix=None):
        Root.__init__(self, f_name, f_shift_data_file, f_ext, f_bias)
        self.f_matrix = f_matrix

    def _main__(self, solution=None):
        problem_size = len(solution)
        if problem_size > 100:
            print("CEC 2005 not support for problem size > 100")
            return 1
        if problem_size == 10 or problem_size == 30 or problem_size == 50:
            self.f_matrix = "rastrigin_M_D" + str(problem_size)
        else:
            print("CEC 2005 F10 function only support problem size 10, 30, 50")
            return 1
        shift_data = self.load_shift_data()[:problem_size]
        matrix = self.load_matrix_data(self.f_matrix)
        z = dot((solution - shift_data), matrix)
        result = 0
        for i in range(0, problem_size - 1):
            result += z[i] ** 2 - 10 * cos(2 * pi * z[i]) + 10
        return result + self.f_bias


